/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
04/2016 - ARDUINO IDE C

This program demonstrates how Arduino IDE can be used to program the Brainbox Arduino - based on the Arduino Leonardo
This program makes minimal use of the Arduino libraries. This brings you closer to real embedded programming.

The Pinout diagram of the BBA (www.e2cre8.be) visualizes the relation between the Arduino pin names and the real pin names

This program  demonstrates how the Analog inputs channels can be programmed in "C"
Use the datasheet of the ATMEGA32U4 from page 298 for more info

We have written 2 functions

READ_ADC_BYTE_CHANNEL: converts the Analog input into a 8 bit digital value - tested for channel 0-7
READ_ADC_INT_CHANNEL: converts the Analog input into a 10 bit digital value - tested for channel 0-7

Connect a potmeter to ADC7 (AN0 ARduino)
to demonstrate the result of the AD conversion is used to change the frequency at the buzzer - connected to E6 on the BBA
*/
  
  
  char ADRESULT = 0;

void setup() {
  // put your setup code here, to run once:
 
  unsigned int READ_ADC_INT_CHANNEL( unsigned char channel ); // function prototype - see below
  unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ); // function prototype - see below
  
  DDRF = 0b00000000; // pin F0 (ARduino pin 23) is digital input (all pins of portF are digital inputs...)
  DDRE = 0b01000000; // pin E6  (ARduino pin 7) connected to buzzer = output pin

}

void loop() {
  // put your main code here, to run repeatedly:
        
        ADRESULT = READ_ADC_BYTE_CHANNEL(7); // convert the analog signal of channel 7 (ARduino A0) into a 8 bit digital signal and store it into "ADRESULT"
      
        PORTE = 0b01000000;     // make pin E6 high
        for (char x = ADRESULT; x>0; x--) // make pin E6 high for as many usec as ADRESULT
        {
        _delay_us(1); 
        }
              //wait
        PORTE = 0b00000000;     // make pin E6 low
        for (char x = ADRESULT; x>0; x--) // // make pin E6 hlow for as many usec as ADRESULT
        {
        _delay_us(1);
        }
}

/*************************************************************************
ADC FUNCTIONS - ADC FUNCTIONS - ADC FUNCTIONS - ADC FUNCTIONS
**************************************************************************/

// returns a 10 bit result from an AD conversion from a certain ADC channel
//Channels 0-7
// A 10 bit result needs to be loaded in an integer variable
unsigned int READ_ADC_INT_CHANNEL( unsigned char channel ){
  
  DDRF &= ~(1<<channel); // config selected channel as input

  // Right adjust + choice of channel
  ADMUX = 0b01000000 +  channel;
  
  // Activate ADC - Stop conversion - prescaler 128 (for ADC 50K<..<200k) (16Mhz / 128 = 125000)
  ADCSRA = 0b10000111;
  
  // start conversion and wait for completion
  ADCSRA |= (1<<ADSC);
  while (ADCSRA & (1<<ADSC));
  
  // return A/D conversion result
  unsigned char ADCLBuff = ADCL; // first read ADCL!!
  return ( ((unsigned int) ADCH ) << 8 ) + ADCLBuff;
}

// returns a 8 bit result from an AD conversion from a certain ADC channel
//Channels 0-7
// A 8 bit result needs to be loaded in a BYTE variable
unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ){
  
  DDRF &= ~(1<<channel); // config selected channel as input

  // Left adjust + choice of channel
  ADMUX = 0b01100000 +  channel ;
  
  // Activate ADC - Stop conversion - prescaler 128 (for ADC 50K<..<200k) (16Mhz / 128 = 125000)
  ADCSRA = 0b10000111;
  
  // start conversion and wait for completion
  ADCSRA |=(1<<ADSC);
  while (ADCSRA & (1<<ADSC));
  
  // return A/D conversion result
  return ADCH;
}

